from functools import wraps
from flask import *
from flask_sqlalchemy import SQLAlchemy
from flask_session import Session
from flask_mail import Mail, Message, Attachment
from flask_login import UserMixin,login_user,login_required,logout_user,current_user,LoginManager
# from flask_recaptcha import ReCaptcha
from flask_toastr import Toastr
from flask_moment import Moment
from pathlib import Path

import os, pymysql
from werkzeug.exceptions import HTTPException
from settings.settings import ConfigFile


app = Flask(__name__, 
            template_folder= 'Templates',
            static_folder='static', 
            instance_relative_config=True,
            instance_path=str(Path('shop').absolute().as_posix()))

app.config.from_object(ConfigFile)

toastr = Toastr(app)
moment = Moment(app)
mail = Mail()
mail.init_app(app)
# recaptcha = ReCaptcha(app)

os.environ["OAUTHLIB_INSECURE_TRANSPORT"] = "1"

db = SQLAlchemy(app)

# engine_container = db.get_engine(app)
# Mwan!k!Andre$1228

app.app_context().push()


# engine_container = db.get_engine(app)

app.config['SESSION_SQLALCHEMY'] = db
Session(app)

login_manager = LoginManager()
login_manager.init_app(app)
# login_manager.login_view = "/login"
login_manager.session_protection = "strong"
login_manager.login_view = "login"
login_manager.login_message_category = "info"

from models.models import *

@app.before_request
def before_request():
    # db.create_all()
    with open('settings/config.json', 'r') as openfile:
        conf = json.load(openfile)
        app.config.update(conf)
        
    # mail.init_app(app)

def cleanup(session):
    session.expunge_all()
    session.close()      

@login_manager.user_loader
def load_user(user_id):
    if user_id:
        return Users.query.get(user_id)
    
@app.errorhandler(HTTPException)
def error_handler(e):
    response = e.get_response()
    agent = request.environ.get('HTTP_USER_AGENT')
    response.data = json.dumps({
        "code": e.code,
        "name": e.name,
        "description": e.description,
        'source': agent
    })
    response.content_type = "application/json"
    return make_response(response, e.code)


@app.route("/migrate", methods=["GET", "POST"])
def make_migrate():
    db.create_all()
    return "DONE"



@app.teardown_request
def teardown_request(exception):
    if exception:
        db.session.rollback()
#     db.session.remove()



