from flask_wtf import *
from wtforms import *
from wtforms.fields.html5 import *
from wtforms.validators import *
from flask_wtf.file import *
# import phonenumbers


# def validate_phone(form, field):
#         if len(field.data) > 16:
#             raise ValidationError('Invalid phone number.')
#         try:
#             input_number = phonenumbers.parse(field.data)
#             if not (phonenumbers.is_valid_number(input_number)):
#                 raise ValidationError('Invalid phone number.')
#         except:
#             input_number = phonenumbers.parse("+1"+field.data)
#             if not (phonenumbers.is_valid_number(input_number)):
#                 raise ValidationError('Invalid phone number.')
            
class login_form(FlaskForm):
    email = StringField(validators=[InputRequired(), Email(), Length(1, 64)])
    pwd = PasswordField(validators=[InputRequired(), Length(min=8, max=72)])
    # Placeholder labels to enable form rendering
    username = StringField(
        validators=[Optional()]
    )
class register_form(FlaskForm):
    username = StringField(
        validators=[
            InputRequired(),
            Length(3, 20, message="Please provide a valid name"),
            Regexp(
                "^[A-Za-z][A-Za-z0-9_.]*$",
                0,
                "Usernames must have only letters, " "numbers, dots or underscores",
            ),
        ]
    )
    # recaptcha = RecaptchaField(validators=[InputRequired()])
    email = StringField(validators=[InputRequired(), Email(), Length(1, 64)])
    pwd = PasswordField(validators=[InputRequired(), Length(8, 72)])
    cpwd = PasswordField(
        validators=[
            InputRequired(),
            Length(8, 72),
            EqualTo("pwd", message="Passwords must match !"),
        ]
    )

class TestimonialForm(FlaskForm):
    name = StringField(validators=[InputRequired()])
    photo = FileField(validators=[FileRequired(),FileAllowed(['jpg', 'png', 'jpeg', 'gif'], 'Images only!')])
    message = TextAreaField(validators=[InputRequired(), Length(1, 140)])
    
class PartnerForm(FlaskForm):
    name = StringField(validators=[InputRequired()])
    photo = FileField(validators=[FileRequired(),FileAllowed(['jpg', 'png', 'jpeg', 'gif'], 'Images only!')])
    description = TextAreaField(validators=[InputRequired(), Length(1, 140)])
    
class CategoryForm(FlaskForm):
    name = StringField(validators=[InputRequired(), Length(4, 80, message="The category name must have between 4 - 80 characters")])
    slug = StringField(validators=[InputRequired()])
    icon = StringField(validators=[InputRequired()])
    description = TextAreaField(validators=[InputRequired(), Length(1, 400)])
    
class ServiceForm(FlaskForm):
    name = StringField(validators=[InputRequired()])
    slug = StringField(validators=[InputRequired()])
    category = SelectField(validators=[InputRequired()], choices=[], validate_choice=False)
    icon = StringField(validators=[InputRequired()])
    description = TextAreaField(validators=[InputRequired()])
    
class SliderForm(FlaskForm):
    caption = StringField(validators=[InputRequired()])
    image = StringField(validators=[InputRequired()])
class ProjectCategoryForm(FlaskForm):
    name = StringField(validators=[InputRequired()])
    description = TextAreaField(validators=[InputRequired()])
    
class ProjectForm(FlaskForm):
    name = StringField(validators=[InputRequired()])
    slug = StringField(validators=[InputRequired()])
    icon = StringField(validators=[InputRequired()])
    category = SelectField(validators=[InputRequired()], choices=[], validate_choice=False)
    description = TextAreaField(validators=[InputRequired()])
   
    
class ShopForm(FlaskForm):
    name = StringField(validators=[InputRequired()])
    price = IntegerField(validators=[InputRequired()])
    cover = FileField(validators=[FileRequired(),FileAllowed(['jpg', 'png', 'jpeg', 'gif'], 'Images only!')])
    product = FileField(validators=[FileRequired(),FileAllowed(['pdf', 'docx', 'doc', 'ppt', 'odt'], 'Documents only!')])
    description = TextAreaField(validators=[InputRequired()])
    
class InquiryForm(FlaskForm):
    name = StringField(validators=[InputRequired()])
    email = StringField(validators=[InputRequired(), Email(), Length(1, 64)])
    phone = StringField(validators=[InputRequired(), Length(min=6, max=40)])
    message = TextAreaField(validators=[InputRequired()])
    
class SubscribeForm(FlaskForm):
    email = StringField(validators=[InputRequired(message='Email address is required'), Email(), Length(1, 64)])
    check = BooleanField(validators=[InputRequired(message="Accept Terms of Use and Privacy Policy")])
    subscription = HiddenField()
    
class BlogForm(FlaskForm):
    title = StringField(validators=[InputRequired()])
    image = HiddenField(validators=[InputRequired(message="Featured Image Required")])
    message = TextAreaField(validators=[InputRequired()])
    
 
class ActivityForm(FlaskForm):
    name =  StringField(validators=[InputRequired(message="Your name is required")])
    email =  StringField(validators=[InputRequired(message="Your email address is required"), 
                                     Email(message="Enter a valid email address"), 
                                     Length(1, 100, message="Invalid address length!")])
    phone = StringField(validators=[InputRequired(message="Your phone number is required"), Length(min=6, max=40, message="Invalid phone number!")])
    activity = HiddenField()

class CareerForm(FlaskForm):
    name =  StringField(validators=[InputRequired(message="Your name is required")])
    email =  StringField(validators=[InputRequired(message="Your email address is required"), 
                                     Email(message="Enter a valid email address"), 
                                     Length(1, 100, message="Invalid address length!")])
    phone = StringField(validators=[InputRequired(message="Your phone number is required"), Length(min=6, max=40, message="Invalid phone number!")])
    document = FileField(validators=[FileRequired(message="Application document(CV, Resume or Application) required"),
                                            FileAllowed(['pdf', 'docx', 'doc', 'odt', 'txt'], 'Documents file only!')
                                            ])
    message = TextAreaField(validators=[InputRequired(message="Cover letter required")])
    # 
class DonationForm(FlaskForm):
          
    name =  StringField(validators=[InputRequired(message="Your name is required")])
    email =  StringField(validators=[InputRequired(message="Your email address is required"), 
                                     Email(message="Enter a valid email address"), 
                                     Length(1, 100, message="Invalid address length!")])
    phone = StringField(validators=[InputRequired(message="Your phone number is required"), Length(min=6, max=40, message="Invalid phone number!")])
    amount =  IntegerField(validators=[DataRequired(message="Please enter your donation amount")])
    
    
class LeaderForm(FlaskForm):
    name =  StringField(validators=[InputRequired(message="Your name is required")])
    email =  StringField(validators=[InputRequired(message="Your email address is required"), 
                                     Email(message="Enter a valid email address"), 
                                     Length(1, 100, message="Invalid address length!")])
    phone = StringField(validators=[InputRequired(message="Your phone number is required"), 
                                    Length(min=6, max=40, message="Invalid phone number!")])
    message = TextAreaField(validators=[InputRequired(message="Message required"), Length(1, 200, message="Invalid Message length!")])
    photo = FileField(validators=[FileRequired(message="Profile Image Required"),FileAllowed(['jpg', 'png', 'jpeg'], 'Images only!')])
    post = StringField(validators=[InputRequired(message="Your role required")])
    twitter = URLField()
    linkedIn = URLField()
    facebook = URLField()
    
    
      
           
    
    

    