
/* Bootstrap 5 JS included */

//console.clear();
//('use strict');


// Drag and drop - single or multiple image files
// https://www.smashingmagazine.com/2018/01/drag-drop-file-uploader-vanilla-js/
// https://codepen.io/joezimjs/pen/yPWQbd?editors=1000
(function () {

  //'use strict';
  
  // Four objects of interest: drop zones, input elements, gallery elements, and the files.
  // dataRefs = {files: [image files], input: element ref, gallery: element ref}

  const preventDefaults = event => {
    event.preventDefault();
    event.stopPropagation();
  };

  const highlight = event =>
    event.target.classList.add('highlight');
  
  const unhighlight = event =>
    event.target.classList.remove('highlight');

  const getInputAndGalleryRefs = element => {
    const zone = element.closest('.upload_dropZone') || false;
    const gallery = zone.querySelector('.upload_gallery') || false;
    const input = zone.querySelector('input[type="file"]') || false;
    return {input: input, gallery: gallery};
  }

  const handleDrop = event => {
    const dataRefs = getInputAndGalleryRefs(event.target);
    dataRefs.files = event.dataTransfer.files;
    handleFiles(dataRefs);
  }

  const Slugify = string => {
    var a = string.split(".");
    if( a.length === 1 || ( a[0] === "" && a.length === 2 ) ) {
        return "";
    }
    var end = a.pop();
    const newText = string
      .toLowerCase()
      .replace(/ /g, "_")
      .replace(/[^\w-]+/g, "");
    
    return newText+'.'+end
    };

  function eventHandlers(zone) {

    const dataRefs = getInputAndGalleryRefs(zone);
    if (!dataRefs.input) return;

    // Prevent default drag behaviors
    ;['dragenter', 'dragover', 'dragleave', 'drop'].forEach(event => {
      zone.addEventListener(event, preventDefaults, false);
      document.body.addEventListener(event, preventDefaults, false);
    });

    // Highlighting drop area when item is dragged over it
    ;['dragenter', 'dragover'].forEach(event => {
      zone.addEventListener(event, highlight, false);
    });
    ;['dragleave', 'drop'].forEach(event => {
      zone.addEventListener(event, unhighlight, false);
    });

    // Handle dropped files
    zone.addEventListener('drop', handleDrop, false);

    // Handle browse selected files
    dataRefs.input.addEventListener('change', event => {
      dataRefs.files = event.target.files;
      handleFiles(dataRefs);
    }, false);

  }


  // Initialise ALL dropzones
  const dropZones = document.querySelectorAll('.upload_dropZone');
  for (const zone of dropZones) {
    eventHandlers(zone);
  }


  // No 'image/gif' or PDF or webp allowed here, but it's up to your use case.
  // Double checks the input "accept" attribute
  const isImageFile = file => 
    ['image/jpeg', 'image/png', 'image/svg+xml'].includes(file.type);

  function convertToSlug(Text) {
    return Text.toLowerCase()
                .replace(/[^\w ]+/g, '')
                .replace(/ +/g, '-');
  }

  function previewFiles(dataRefs) {
    if (!dataRefs.gallery) return;
    $('.upload_text').addClass('visually-hidden')
    $('.up-btn').addClass('visually-hidden')
    $('.btn-upload-more').removeClass('visually-hidden')

    for (const file of dataRefs.files) {
      var id = dataRefs.files.indexOf(file);
      let reader = new FileReader();
      reader.readAsDataURL(file);
      reader.onloadend = function() {
        var img = `
            <div class='col-2 mt-2 id`+id+`'>
            <input class="check-image float-end position-absolute" type="checkbox" id="`+convertToSlug(file.name)+`" name="position-absolute" onchange = "Checked('`+Slugify(file.name)+`')" value="{{url_for('static', filename='blog')}}/`+Slugify(file.name)+`">
              <label for="`+convertToSlug(file.name)+`">
                <img class="img-fluid img`+id+`" src="`+reader.result+`"  alt='`+file.name+`'>
              
                <div class="progress rounded-0 mb-0 " style="height:5px !important;">
                    <div class="progress-bar bg-danger rounded-0 prog" id="p_id`+convertToSlug(file.name)+`" role="progressbar" aria-label="Example with label" style="width: 0%; font-size:6px;" aria-valuenow="0" aria-valuemin="0" aria-valuemax="100"></div>
                </div>
              </label>
            </div>
        `

        $('.upload_gallery').append(img)
        const regex = /data:.*base64,/
        var node = $("#p_id"+convertToSlug(file.name))
        $.ajax({
            xhr: function() {
                var xhr = new window.XMLHttpRequest();
                xhr.upload.addEventListener("progress", function(evt) {
                  if (evt.lengthComputable) {
                    var percentComplete = evt.loaded / evt.total;
                    percentComplete = parseInt(percentComplete * 100);
                    node.css('width', percentComplete+"%")
                    node.attr('aria-valuenow', percentComplete)
                    if (percentComplete == 100){
                        node.removeClass('bg-danger').addClass('bg-success');
                    }
                  }
                }, false);
            
                return xhr;
              },
              url: "/admin/blog/upload_files",
              type: 'POST',
              data: {image_upload: reader.result.replace(regex,""), name: Slugify(file.name)},
              success: function(response){
                console.log('posted: ', response);
              }
        })
      }
    }
  }

  // Based on: https://flaviocopes.com/how-to-upload-files-fetch/
  const imageUpload = dataRefs => {

    // Multiple source routes, so double check validity
    if (!dataRefs.files || !dataRefs.input) return;

    const url = dataRefs.input.getAttribute('data-post-url');
    if (!url) return;

    const name = dataRefs.input.getAttribute('data-post-name');
    if (!name) return;

    const formData = new FormData();
    formData.append(name, dataRefs.files);

    console.log(dataRefs.files.length)

    // reworking this part to work with ajax
    for (var fil of dataRefs.files){
        console.log($('.img'+dataRefs.files.indexOf(fil)))
        var node = $(document).find(".p_id"+dataRefs.files.indexOf(fil))

        var formD = new FormData();
        formD.append('upoad_image', fil);
        $.ajax({
            xhr: function() {
                var xhr = new window.XMLHttpRequest();
                xhr.upload.addEventListener("progress", function(evt) {
                  if (evt.lengthComputable) {
                    var percentComplete = evt.loaded / evt.total;
                    percentComplete = parseInt(percentComplete * 100);
                    node.css('width', percentComplete+"%")
                    node.attr('aria-valuenow', percentComplete)
                  }
                }, false);
            
                return xhr;
              },
              url: url,
              type: 'POST',
              processData : false,
              data: {image_upoad: $('.img'+dataRefs.files.indexOf(fil)).attr('src')},
              success: function(response){
                console.log('posted: ', response);
              }
        })


    }

    //fetch(url, {
    //  method: 'POST',
    //  body: formData
    //})
    //.then(response => response.json())
    //.then(data => {
    //  console.log('posted: ', data);
    //  if (data.success === true) {
    //    ////previewFiles(dataRefs);
    //  } else {
    //    //console.log('URL: ', url, '  name: ', name)
    //  }
    //})
    //.catch(error => {
    //  console.error('errored: ', error);
    //});
    //------------end here
  }


  // Handle both selected and dropped files
  const handleFiles = dataRefs => {

    let files = [...dataRefs.files];

    // Remove unaccepted file types
    files = files.filter(item => {
      if (!isImageFile(item)) {
        console.log('Not an image, ', item.type);
      }
      return isImageFile(item) ? item : null;
    });

    if (!files.length) return;
    dataRefs.files = files;

    previewFiles(dataRefs);
    //imageUpload(dataRefs);
  }

})();

$.ajax({
    url: "/admin/blog/blog_images",
    success: function(response){
        for( image of response.images){
            var id = response.images.indexOf(image);
            var img = `
            <div class='col-2 mt-2 id`+id+`'>
            <input class="check-image float-end position-absolute" id="ii`+id+`" type="checkbox" name="position-absolute" onchange = "Checked('`+image+`')" value="{{url_for('static', filename='blog')}}/`+name+`">
            <label for="ii`+id+`"><img class="img-fluid img`+id+`" src="/static/blog/`+image+`"  alt='`+image+`'></label>
            </div>
        `
        $('#image_gallery').append(img)
        }
    }
})