from __init__ import *
from forms.forms import *
from werkzeug.security import generate_password_hash, check_password_hash
from werkzeug.utils import secure_filename, send_file, send_from_directory
from wtforms.validators import *
from random import randrange, shuffle, random
import requests, jwt, json, glob, os
from sqlalchemy import func, inspect, or_
from mpesa.mpesa import FlaskMPesa

def token_required(f):
    @wraps(f)
    def decorated(*args, **kwargs):
        token = None
        if 'token' in request.args:
            token = request.args['token']
            try:
                data = jwt.decode(token, app.config['SECRET_KEY'], algorithms=["HS256"])
                books = data.get('books')
                return f(books, *args, **kwargs)
            except Exception as e:
                return jsonify({'message' : str(e), 'success': False}), 401
        if not token:
            return jsonify({'message' : 'Broken link', 'success': False}), 401 
    return decorated

def special_requirement(f):
    @wraps(f)
    def wrap(*args, **kwargs):
        try:
            if current_user.email:
                return f(*args, **kwargs)
            else:
                flash('Not Auuthorized to view this file', 'warning')
                return redirect(url_for('index'))
        except Exception as e:
            print(e)
            flash('Error handling your request', 'error')
            return redirect(url_for('index'))
    return wrap

def user_login(f):
    @wraps(f)
    def decorated(*args, **kwargs):
        user = None
        if 'user' in session:
            user = session.get('user')
            if user:
                try:
                    return f(user, *args, **kwargs)
                except Exception as e:
                    return jsonify({'login': True, 'message': str(e), 'success': False})
            else:
                if request.path == url_for('checkout'):
                    return redirect(f"/login?next={request.path}")
                else:
                    return jsonify({'login': True, 'message': 'Unauthenticated', 'success': False})
        if not user:
            if request.path == url_for('checkout'):
                return redirect(f"/login?next={request.path}")
            else:
                return jsonify({'login': True, 'message': 'Unauthenticated', 'success': False})
    return decorated


def object_as_dict(obj):
    obj = obj._asdict()
    return obj


def validate_email(email):
    if Users.query.filter_by(email=email).first():
        raise ValidationError("Email already registered!")
    else:
        return email


def base64ToPNG(bytefile, name):
    import base64
    with open(f"static/blog/{name}", "wb") as fh:
        fh.write(base64.decodebytes(bytefile))
    return f"{name}"


def validate_uname(uname):
    if Users.query.filter_by(username=uname).first():
        raise ValidationError("Username already taken!")
    else:
        return uname


def Slugify(string):
    string = re.sub('[^A-Za-z0-9]+', ' ', string).replace(" ", "_").lower()
    return string


def unique_slug(classname, slug):
    slug = f"noredge_{Slugify(slug)}"
    if classname.query.filter_by(slug=slug).first():
        # return slug+"_"+str(randrange(1,100))
        raise ValidationError("This item already exists")
    else:
        return slug


def allCaps(string):
    string = re.sub('[^A-Za-z0-9]+', ' ', string).replace(" ", "_").upper()
    return string


def Returner(page):
    form = InquiryForm()
    subscribe = SubscribeForm()
    return make_response(render_template(f'app/{page}.html', form=form, subscribe=subscribe))


def UploadsProfileImage(f):
    filename = secure_filename(f.filename)
    f.save(os.path.join(app.config['UPLOAD_FOLDER'], 'profile', filename))
    return filename


def UploadsImage(f):
    filename = secure_filename(f.filename)
    f.save(os.path.join(app.config['UPLOAD_FOLDER'], 'uploads', filename))
    return filename


def UploadProduct(f):
    filename = secure_filename(f.filename)
    f.save(os.path.join(app.instance_path, filename))
    return filename

def UploadDocuments(f):
    filename = secure_filename(f.filename)
    f.save(os.path.join(app.config['DOCUMENTS_FOLDER'], filename))
    return filename

@token_required
def download(books):
    for filename in books:
        try:
            p = send_from_directory(os.path.join(app.instance_path, ''), filename)
            # flash('Item sent to your email account', 'info')
            return p
        except Exception as e:
            # flash('File not found', 'info')
            # return redirect(url_for('index'))
            print('Failed')
            

@token_required
def protected_folder(books):
    print(books)
    return redirect(url_for('index'))

@token_required
def protected():
    print('Downloading')

def index():
    form = InquiryForm()
    subscribe = SubscribeForm()
    testimonials = db.session.query(Testimonials).filter(
        Testimonials.published == True).all()
    services = db.session.query(ServiceCategories).filter(
        ServiceCategories.active == True).order_by(ServiceCategories.id.desc()).all()
    posts = db.session.query(Posts.id, Posts.active, Posts.keywords,
                             Posts.title, Posts.slug, Posts.image, Posts.upload_date, Posts.content,
                             Users.email).outerjoin(Users, Users.id == Posts.user_id).order_by(Posts.id.desc()).limit(4).all()
    # cleanup(db.session)
    return make_response(
        render_template(
            'app/index.html',
            form=form,
            subscribe=subscribe,
            testimonials=testimonials,
            services=services,
            posts=posts
        ))
def activities(slug):
    form = ActivityForm()
    project = Activity.query.filter_by(slug = slug).first()
    if form.validate_on_submit():
        print(request.form)
        new = ActivityRequests(
            name = form.name.data,
            email = form.email.data,
            phone = form.phone.data,
            activity = form.activity.data
        )
        db.session.add(new)
        db.session.commit()
        flash('Request sent successfully', 'success')
    return render_template('fr_app/activity.html', project = project, form = form)

def search_getinvolved():
    projects = db.session.query(Activity.name, Activity.slug, Activity.id).filter_by(active = True).all()
    pr = []
    for p in projects:
        pr.append(object_as_dict(p))
    return make_response(jsonify({'projects': pr}))

def search_services():
    projects = db.session.query(Projects.name, Projects.slug, Projects.id).filter_by(active = True).all()
    pr = []
    for p in projects:
        pr.append(object_as_dict(p))
    return make_response(jsonify({'projects': pr}))


def CartItems():
    items = []
    total_cost = 0
    for item in session.get('cart'):
        items.append(object_as_dict(db.session.query(Products.p_id,
                                                     Products.name,
                                                     Products.currency,
                                                     Products.price,
                                                     Products.cover,
                                                     Products.description).filter(Products.p_id == item).first()))
    for price in items:
        total_cost += price.get('price')

    return (items, total_cost)

# @user_login


def add_to_cart():
    item_id = request.values.get('item_id')

    if 'cart' not in session:
        session['cart'] = [item_id]
        list1 = [item_id]
        total_cost = 0

        if 'user' in session:
            items_in_db = db.session.query(Carts.items).filter(
                Carts.user == session.get('user').id, Carts.checkedout == False).first()
            if items_in_db:
                items_in_db = object_as_dict(items_in_db)
                list2 = list(set(items_in_db.extend(list1)))
                session['cart'] = list2
            else:
                new = Carts(
                    items=list1,
                    user=session.get('user').id
                )
                try:
                    db.session.add(new)
                    db.session.commit()
                except Exception as e:
                    print(e)

        items, total_cost = CartItems()

        return make_response(jsonify({'success': True,
                                      "message": "Item succesfuly added to the cart",
                                      'items_count': len(session.get('cart')),
                                      'items': items,
                                      'total_cost': total_cost}))
    if 'cart' in session:
        if item_id in session['cart']:
            return make_response(jsonify({'success': False,
                                          "message": "Item already in the cart",
                                          'items_count': len(session.get('cart'))}))
        session.get('cart').append(item_id)
        list1 = session.get('cart')
        total_cost = 0

        if 'user' in session:
            items_in_db = Carts.query.filter_by(
                user=session.get('user').id, checkedout=False).first()
            if items_in_db:
                list2 = list(set(session['cart']) | set(items_in_db.items))
                session['cart'] = list2
                items_in_db.items = list2
                db.session.commit()

            else:
                new = Carts(
                    items=list1,
                    user=session.get('user').id
                )
                try:
                    db.session.add(new)
                    db.session.commit()
                except Exception as e:
                    print(e)

        items, total_cost = CartItems()

        return make_response(jsonify({'success': True,
                                      "message": "Item succesfuly added to the cart",
                                      'items_count': len(session.get('cart')),
                                      'items': items,
                                      'total_cost': total_cost}))


def remove_from_cart():
    p_id = request.values.get('p_id')
    session['cart'].remove(p_id)
    if 'user' in session:
        items_in_db = Carts.query.filter_by(
            user=session.get('user').id, checkedout=False).first()
        if items_in_db:
            items_in_db.items = session.get('cart')
            db.session.commit()

    return make_response(jsonify({"success": True}))


def show_cart():
    items, total_cost = CartItems()
    return make_response(jsonify({'success': True,
                                  "message": "This is the cart",
                                  'items_count': len(session.get('cart')),
                                  'items': items,
                                  'total_cost': total_cost}))


@user_login
def logout_user(user):
    session.pop('user')
    if 'cart' in session:
        session.pop('cart')
    return redirect(url_for('our_program'))


@user_login
def checkout(user):
    form = InquiryForm()
    subscribe = SubscribeForm()
    items, total_cost = CartItems()
    return make_response(render_template('app/checkout.html',
                                         form=form,
                                         subscribe=subscribe,
                                         items_count=len(session.get('cart')),
                                         items=items,
                                         currency='KES',
                                         total_cost=total_cost))


@user_login
def submit_mpesa(user):
    # def submit_mpesa():
    values = request.values
    phone = values.get('phone')

    items, cost = CartItems()

    cart_id = db.session.query(Carts.id).filter(
        Carts.user == session.get('user').id, Carts.checkedout == False).first()

    result = FlaskMPesa(MpesaAccessToken, phone, cost).lipa_na_mpesa_online()
    if result.get('ResponseCode') == '0':
        req = MpesaPaymentsRequests(
            email=session.get('user').email,
            user_id=session.get('user').id,
            CheckoutRequestID=result.get('CheckoutRequestID'),
            CustomerMessage=result.get('CustomerMessage'),
            MerchantRequestID=result.get('MerchantRequestID'),
            ResponseCode=result.get('ResponseCode'),
            ResponseDescription=result.get('ResponseDescription')
        )
        db.session.add(req)
        db.session.commit()
        # cleanup(db.session)
        return jsonify({'status': 'success', "message": result.get('ResponseDescription')})
    else:
        # cleanup(db.session)
        return jsonify({'status': 'error', "message": result.get('errorMessage')})
@user_login
def register_urls():
    phone = "0702109810"

    items, cost = CartItems()
    result = FlaskMPesa.register_urls()
    print(result)
    
@user_login
def checkout_cart(user):
    items = session.get('cart')
    session.pop('cart')
    print(items)
    items_in_db = Carts.query.filter_by(
        user=session.get('user').id, checkedout=False).first()
    items_in_db.checkedout = True
    db.session.commit()
    return make_response(jsonify({'success': True, "message": "Cart items cleared"}))

def mailsender():
    return MailSenderFn(1, 8)


def MailSenderFn(cart_id, user_id):
    # print(mail)
    cart = Carts.query.filter_by(id=cart_id).first()
    user = Users.query.filter_by(id=user_id).first()
    name = user.username
    email = user.email
    host = "https://noredge.carnasi.com"
    book = str(tuple(cart.items))
    files = []
    n = ""
    for i in cart.items:
        p = Products.query.filter_by(p_id = i).first()
        x = {
            "file": p.link,
            "filename": p.name
        }
        files.append(x)
        n += p.name+", "
    data = {
            'books': cart.items,
            'name': user.username,
            'cost': cart.cost
            }
    token = jwt.encode(data, app.config['SECRET_KEY'], algorithm="HS256")
    try:
        msg = Message()
        msg.subject = f"{app.config.get('PROJECT_NAME')} program"
        msg.sender = (app.config.get('PROJECT_NAME'),"noreply@reply.autolink.co.ke")
        msg.recipients = ["cyrilletony98@gmail.com"]
        msg.body = f"""
        Dear Valued {name}, \n
        We hope this email finds you well. We would like to express our appreciation for your continued support of Noredge Economics Group program. To access the book you purchase, please find the attachment named {n} and simply click on it to open. In the event that the attachment does not work, we have also provided a link to download the book below:
        {host}/download?token={token} \n
        We hope that you find this book to be informative and engaging. If you have any feedback or questions, please feel free to reach out to our customer support team at {host}. We always appreciate hearing from our customers and value your input. Thank you for choosing Noredge Economics Group program as your preferred platform for economic literature. We look forward to continuing to provide you with high-quality content in the future. Best regards, [Insert Name] Noredge Economics Group program Team"""
        # msg.html = render_template('mails/program.html',
        #                         host = host,
        #                         linked_token = linked_token)

        # attachments = [Attachment(filename=secure_filename(file), content_type="application/pdf", data=file.read()) for file in files]
        attachments = []
        for file in files:
            with app.open_resource(f"program/{file.get('file')}") as fp:
                attachments.append(Attachment(filename=file.get('filename'), content_type="application/pdf", data=fp.read()))
        msg.attachments = attachments
        mail.send(msg)

        return make_response(jsonify({'message': "Message sent successfuly"}), 201)
    except Exception as e:
        print(e)
        return make_response(jsonify({'message': "Error sending mail"}), 201)

def getGallery():
    projects = db.session.query(Projects.name, Projects.slug, Projects.id, Projects.images).filter_by(active = True).all()
    pr = []
    for p in projects:
        
        for img in p.images:
            pr2 = {'name' : p.name,'image': img, 'slug': p.slug}
            pr.append(pr2)
    shuffle(pr)
    pr = pr[0:4]
    return pr
def blogPosts():
    posts = db.session.query(Posts.id, Posts.active, Posts.keywords,
                             Posts.title, Posts.slug, Posts.image, Posts.upload_date, Posts.content,
                             Users.email)\
                                .outerjoin(Users, Users.id == Posts.user_id)\
                                .filter(Posts.active == True)\
                                .order_by(Posts.id.desc())\
                                .all()
    return posts

def index2():
    pr = getGallery()
    blog = blogPosts()
    posts = blog[0:3]
    sliders = SliderModel.query.filter_by(active = True).all()
    projects = Projects.query.filter_by(active = True).order_by(Projects.id.desc()).limit(6).all()
    return render_template('fr_app/index.html', projects = projects, pr = pr, posts = posts, sliders = sliders)

@login_required
def leader_add():
    form = LeaderForm()
    if form.validate_on_submit():
        new = Leaders(
            image = UploadsProfileImage(form.photo.data),
            name = form.name.data,
            phone = form.phone.data,
            email = form.email.data,
            post = form.post.data,
            message = form.message.data,
            facebook = form.facebook.data,
            twitter = form.twitter.data,
            linkedIn = form.linkedIn.data,
        )
        db.session.add(new)
        db.session.commit()
        
        
    return render_template('admin/noredge/add_leader.html', form = form)

@login_required
def admin_leadership():
    leaders = Leaders.query.all()
    return render_template('admin/noredge/leadership.html', leaders = leaders)

def leaders_list():
    leaders = db.session.query(
                                Leaders.id,
                                Leaders.name,
                                Leaders.phone,
                                Leaders.email,
                                Leaders.post,
                                Leaders.message,
                                Leaders.image,
                                Leaders.facebook,
                                Leaders.linkedIn,
                                Leaders.twitter,
                                Leaders.upload_date,
                                Leaders.active
                               ).filter_by(active=False).limit(3).all()
    l = []
    for ll in leaders:
        l.append(object_as_dict(ll))
    
    return make_response(jsonify(l))

@login_required
def request_approve(id):
    data = CareersModel.query.filter_by(id=id).first()
    data.active = False if data.active else True
    db.session.commit()
    return redirect(url_for('all_career_requests'))

@login_required
def all_career_requests():
    requests = CareersModel.query.order_by(CareersModel.id.desc()).all()
    #filter_by(active = True).
    return make_response(render_template('admin/noredge/careers.html', requests = requests))

def careers():
    form = CareerForm()
    pr = getGallery()
    if form.validate_on_submit():
        
        new = CareersModel(
            name = form.name.data,
            email = form.email.data,
            phone = form.phone.data,
            message = form.message.data,
            document = UploadDocuments(form.document.data)
        )
        db.session.add(new)
        db.session.commit()
        flash('Details submitted successfully', 'success')
    return render_template('fr_app/careers.html', form = form, pr = pr)

def about2():
    pr = getGallery()
    return render_template('fr_app/about.html', pr = pr)

def projects(slug):
    project = Projects.query.filter_by(slug = slug).first()
    return render_template('fr_app/projects.html', project = project)

def contact2():
    form = InquiryForm()
    if request.method == "POST":
        if form.validate_on_submit():
            new = Inquiries(
                name = form.name.data,
                email = form.email.data,
                phone = form.phone.data,
                message = form.message.data
            )
            db.session.add(new)
            db.session.commit()
            flash('Inquiry sent successfully', 'success')
    return render_template('fr_app/contact.html', form = form)

def blog2():
    posts = blogPosts()
    return render_template('fr_app/blog.html', posts = posts)

def submit_donation():
    # form = DonationForm(request.POST, )
    # if form.validate_on_submit():
    #     print(form)
    #     if request.method == "POST":
    #         form = request.form
    #         print(form)
    # form.amount1.choices = [(1000, 'Ksh 1000'), (2000, 'Ksh 2000'), (5000, 'Ksh 5000')]
    return redirect(url_for('donate_mbele_foundation'))

def ValidPhoneNumber(phone):
    phone = str(phone.replace(" ", ""))
    if phone[0] == "0":
        phone = f"254{phone[1:]}"
    elif phone[0] == '+':
        phone = str(phone[1:]) 
    return phone

def donate():
    pr = getGallery()
    form = DonationForm()
    if request.method == "POST":
        if form.validate_on_submit():
            data = request.form
            # print(data)
            new = Donations(
                name = form.name.data,
                email = form.email.data,
                amount = form.amount.data,
                phone = ValidPhoneNumber(form.phone.data)
            )
            db.session.add(new)
            db.session.commit()
            result = FlaskMPesa(MpesaAccessToken, new.phone, new.amount).lipa_na_mpesa_online()
            if result.get('ResponseCode') == '0' or result.get('ResponseCode') == 0:
                req = MpesaPaymentsRequests(
                    email=new.email,
                    user_id= new.id,
                    CheckoutRequestID=result.get('CheckoutRequestID'),
                    CustomerMessage=result.get('CustomerMessage'),
                    MerchantRequestID=result.get('MerchantRequestID'),
                    ResponseCode=result.get('ResponseCode'),
                    ResponseDescription=result.get('ResponseDescription')
                )
                db.session.add(req)
                db.session.commit()
                flash(result.get('ResponseDescription'), 'success')
            else:
                flash(result.get('errorMessage'), 'error')
            
    return render_template('fr_app/donate.html', form = form, pr = pr)

def gallery():
    projects = db.session.query(Projects.name, Projects.slug, Projects.id, Projects.images).filter_by(active = True).all()
    pr = []
    for p in projects:
        
        for img in p.images:
            pr2 = {'name' : p.name,'image': img, 'slug': p.slug}
            pr.append(pr2)
    return render_template('fr_app/gallery.html', images = pr)
    
    
def mpesa():
    if request.method == 'POST':
        r = request.json

        with open("sample.json", "w") as outfile:
            json.dump(r, outfile)

        result = r.get('Body').get('stkCallback')
        call = MpesaPaymentsRequests.query.filter_by(
            CheckoutRequestID=result.get('CheckoutRequestID')).first()
        if result.get('ResultCode') == 0:
            new = PaymentsTable(
                payment_type='MPESA',
                email=call.email,
                user_id=call.user_id,
                checkoutid=result.get('CheckoutRequestID'),
                merchantid=result.get('MerchantRequestID'),
                resultcode=result.get('ResultCode'),
                resultdesc=result.get('ResultDesc'),
                amount=result.get('CallbackMetadata').get('Item')[0]['Value'],
                currency="KES",
                receipt_number=result.get('CallbackMetadata').get('Item')[1]['Value'],
                phone_number=result.get('CallbackMetadata').get('Item')[ 4]['Value']

            )
            db.session.add(new)
            db.session.commit()
            MailSenderFn(call.cart_id, call.user_id)
            # cleanup(db.session)
            return jsonify({'Payment': 'Success'})
        else:
            new = MpesaErrors(
                user_id=call.user_id,
                checkoutid=result.get('CheckoutRequestID'),
                merchantid=result.get('MerchantRequestID'),
                resultcode=result.get('ResultCode'),
                resultdesc=result.get('ResultDesc'),
            )
            db.session.add(new)
            db.session.commit()
            # cleanup(db.session)
            return jsonify({'Payment': 'Error'})
    return jsonify({'payment': 'API'})


def partnerships():
    return Returner('partnerships')


def buy_book(p_id):
    form = InquiryForm()
    subscribe = SubscribeForm()
    book = object_as_dict(db.session.query(Products.p_id,
                                           Products.name,
                                           Products.currency,
                                           Products.price,
                                           Products.cover,
                                           Products.description).filter(Products.p_id == p_id).first())
    return make_response(render_template(f'app/buy_book.html', form=form, subscribe=subscribe, book=book))


def our_program():
    form = InquiryForm()
    subscribe = SubscribeForm()
    items = db.session.query(Products).order_by(
        func.random()).filter(Products.active == True).all()
    if 'user' in session:
        items_in_db = Carts.query.filter_by(
            user=session.get('user').id, checkedout=False).first()
        if items_in_db:
            session['cart'] = items_in_db.items
        else:
            session['cart'] = []
    return make_response(render_template('app/our_program.html', form=form, subscribe=subscribe, items=items))


def region():

    region = requests.get("http://ip-api.com/json").json()
    country = Country.query.filter_by(iso=region.get('countryCode')).first()
    j = []
    for c in Country.query.all():
        j.append({'countryName': c.name,
                  'ISO': c.iso,
                  'ISO3': c.iso3,
                  'countryCode': c.dial,
                  'currency': c.currency,
                  'currencyName': c.currency_name})

    j = json.dumps(j)
    print(j)
    amount = 1200000
    current_currency = "KES"
    p = "0wheInt3I3xUZma41tBjVy8sz2okbephOBuXJD37"
    response = requests.get(f"https://api.apilayer.com/fixer/convert?to={country.currency}&from={current_currency}&amount={amount}",
                            headers={"apikey": "xeBNLL6iDgBTnsvH4OKHfhw3s25XCJpP"})
    # cleanup(db.session)
    result = response.json()

    return make_response(jsonify(result))


def about():
    form = InquiryForm()
    subscribe = SubscribeForm()
    partners = Partners.query.filter(Partners.published == True).order_by(
        Partners.upload_date.desc()).all()
    # cleanup(db.session)
    return make_response(render_template('app/about.html', form=form, subscribe=subscribe, partners=partners))


def blog():
    form = InquiryForm()
    subscribe = SubscribeForm()
    posts = db.session.query(Posts.id, Posts.active, Posts.keywords,
                             Posts.title, Posts.slug, Posts.image, Posts.upload_date, Posts.content,
                             Users.email).outerjoin(Users, Users.id == Posts.user_id).filter(Posts.active == True).order_by(Posts.id.desc()).all()
    # cleanup(db.session)
    return make_response(render_template('app/blog.html', form=form, subscribe=subscribe, posts=posts))


def post(slug):
    form = InquiryForm()
    subscribe = SubscribeForm()
    post = db.session.query(Posts.id, Posts.active, Posts.keywords,
                            Posts.title, Posts.slug, Posts.image, Posts.upload_date, Posts.content,
                            Users.email).outerjoin(Users, Users.id == Posts.user_id).filter(Posts.slug == slug).first()
    # cleanup(db.session)
    return make_response(render_template('app/post.html', form=form, subscribe=subscribe, post=post))


def contacts():
    return Returner('contacts')


def services():
    return Returner('services')


def migrate():
    db.create_all()
    return "DOne"


def service_page(slug):
    form = InquiryForm()
    subscribe = SubscribeForm()
    service = db.session.query(ServiceCategories).filter(
        ServiceCategories.slug == slug).first()
    services = db.session.query(AllServices).filter(
        AllServices.service_category == service.id).all()
    # cleanup(db.session)
    return make_response(render_template('app/service_page.html', form=form, subscribe=subscribe, service=service, services=services))


def send_inquiry():
    form = InquiryForm()
    if request.method == "POST":
        if form.validate_on_submit():
            new = Inquiries(
                name=form.name.data,
                message=form.message.data,
                phone=form.phone.data,
                email=form.email.data,
                service=form.service.data
            )
            try:
                db.session.add(new)
                db.session.commit()

                flash('Inquiry sent successfully!', 'success')
            except Exception as e:
                flash(e, 'error')
            # finally:
                # cleanup(db.session)

    return redirect(form.path.data)


def send_subscription():
    subscribe = SubscribeForm()
    if request.method == "POST":
        if subscribe.subscription:
            if subscribe.validate_on_submit():
                new = Subscribers(
                    email=subscribe.email.data
                )
                try:
                    db.session.add(new)
                    db.session.commit()
                    flash(
                        f"Subscrition to {app.config.get('PROJECT_NAME')} is successfull", 'success')
                except Exception as e:
                    flash(e, 'error')
                finally:
                    cleanup(db.session)

    return redirect(url_for('index'))


def login():
    form = login_form()
    if form.validate_on_submit():
        try:
            user = Users.query.filter_by(email=form.email.data).first()
            if user:
                if check_password_hash(user.pwd, form.pwd.data):
                    if user.roles == "admin":
                        login_user(user)
                        if request.args.get('next'):
                            return redirect(request.args.get('next'))
                        else:
                            return redirect(url_for('dashboard'))
                    elif user.roles == 'user':
                        session['user'] = user
                        if request.args.get('next'):
                            return redirect(request.args.get('next'))
                        else:
                            return redirect(url_for('our_program'))
                else:
                    flash("Invalid Username or password!", "danger")
            else:
                flash("User does not exist!", "danger")
        except Exception as e:
            flash(e, "danger")
        finally:
            cleanup(db.session)
    return make_response(render_template('app/auth.html', form=form))


def register():
    form = register_form()
    if form.validate_on_submit():
        try:
            email = validate_email(form.email.data)
            pwd = form.pwd.data
            username = validate_uname(form.username.data)

            newuser = Users(
                username=username,
                email=email,
                pwd=generate_password_hash(pwd),
            )

            db.session.add(newuser)
            db.session.commit()
            flash(f"Account Succesfully created", "success")
            return redirect(url_for("login"))

        except Exception as e:
            flash(e, "danger")
        # finally:
        #     cleanup(db.session)
    return make_response(render_template('app/auth.html', form=form))

# Admin Dashboard


@login_required
def dashboard():
    print(current_user.email)
    return make_response(render_template('admin/noredge/index.html'))
# ===================

# Admin Views


@login_required
def blank():
    return render_template('admin/noredge/blank.html')


@login_required
def inquiries():
    inquiries = db.session.query(Inquiries).order_by(Inquiries.id.desc()).all()
    # cleanup(db.session)
    return render_template('admin/noredge/inquiries.html', inquiries=inquiries)


@login_required
def inquiries_sort(id):
    data = Inquiries.query.filter_by(id=id).first()
    data.handled = False if data.handled else True
    db.session.commit()
    # cleanup(db.session)
    return redirect(url_for('inquiries'))


@login_required
def inquiries_delete(id):
    data = Inquiries.query.filter_by(id=id).first()
    db.session.delete(data)
    db.session.commit()
    # cleanup(db.session)
    return redirect(url_for('inquiries'))


@login_required
def partners():
    form = PartnerForm()

    if form.validate_on_submit():
        if request.method == 'POST':
            values = request.form
            filename = UploadsProfileImage(form.photo.data)
            try:
                new = Partners(
                    name=values.get('name'),
                    uploader=current_user.id,
                    description=values.get('description'),
                    image=filename
                )

                db.session.add(new)
                db.session.commit()
                flash('Testimonial added succefully', 'success')
            except Exception as e:
                flash('Error adding testimonial', 'error')
            # finally:
                # cleanup(db.session)
    data = Partners.query.order_by(Partners.upload_date.desc()).all()
    return render_template('admin/noredge/partners.html', form=form, data=data)


@login_required
def sliders():
    form = SliderForm()
    if form.validate_on_submit():
        new = SliderModel(
            caption = form.caption.data,
            image = form.image.data
        )
        db.session.add(new)
        db.session.commit()
        flash('Slider Added Successfully')
    sliders = SliderModel.query.all()
    return render_template('admin/noredge/sliders.html', form = form, sliders= sliders)

@login_required
def slider_publish(id):
    print('slider_publish')
    data = SliderModel.query.filter_by(id=id).first()
    data.active = False if data.active else True
    db.session.commit()
    # cleanup(db.session)
    return redirect(url_for('sliders'))
    
@login_required
def slider_delete(id):
    data = SliderModel.query.filter_by(id=id).first()
    db.session.delete(data)
    db.session.commit()
    # cleanup(db.session)
    return redirect(url_for('sliders'))

@login_required
def testimonials_publish(id):
    data = Testimonials.query.filter_by(id=id).first()
    data.published = False if data.published else True
    db.session.commit()
    # cleanup(db.session)
    return redirect(url_for('testimonials'))


@login_required
def testimonials_delete(id):
    data = Testimonials.query.filter_by(id=id).first()
    db.session.delete(data)
    db.session.commit()
    # cleanup(db.session)
    return redirect(url_for('testimonials'))


@login_required
def partners_publish(id):
    data = Partners.query.filter_by(id=id).first()
    data.published = False if data.published else True
    db.session.commit()
    # cleanup(db.session)
    return redirect(url_for('partners'))


@login_required
def partners_delete(id):
    data = Partners.query.filter_by(id=id).first()
    db.session.delete(data)
    db.session.commit()
    # cleanup(db.session)
    return redirect(url_for('partners'))


@login_required
def testimonials():
    form = TestimonialForm()

    if form.validate_on_submit():
        if request.method == 'POST':
            values = request.form
            filename = UploadsProfileImage(form.photo.data)
            try:
                new = Testimonials(
                    name=values.get('name'),
                    uploader=current_user.id,
                    message=values.get('message'),
                    image=filename
                )

                db.session.add(new)
                db.session.commit()
                flash('Testimonial added succefully', 'success')
            except Exception as e:
                flash('Error adding testimonial', 'error')
            # finally:
                # cleanup(db.session)
    data = Testimonials.query.order_by(Testimonials.upload_date.desc()).all()
    return render_template('admin/noredge/testimonials.html', form=form, data=data)


@login_required
def test(url):
    return render_template(f'admin/pages/{url}.html')

# =====================

# About us admin Views


@login_required
def about_us():
    return render_template('admin/noredge/aboutus/about_us.html')


@login_required
def our_socials():
    return render_template('admin/noredge/aboutus/our_socials.html')

# ====================
# program views


@login_required
def activity_add():
    form = ProjectForm()
    if request.method == "POST":
        if form.validate_on_submit():
            try:
                new = Activity(
                    name=form.name.data,
                    description=form.description.data,
                    icon = form.icon.data
                )
                db.session.add(new)
                db.session.commit()
                flash('Project added succefully', 'success')
            except Exception as e:
                flash('Error adding project', 'error')
    return render_template('admin/noredge/activities/add_activity.html', form=form)
@login_required
def activity_items():
    items = Activity.query.all()
    if request.method == 'POST':
        project = Activity.query.filter_by(id = request.form.get('project_id')).first()
        project.images = list(set(project.images + request.form.getlist('images')))
        db.session.commit()
    return render_template('admin/noredge/activities/all_activities.html', items=items)

def mbele_blog_post(slug):
    post = Posts.query.filter_by(slug=slug).first()
    others = Posts.query.order_by(func.random()).limit(4).all()
    return render_template('fr_app/article.html', post = post, others = others)

@login_required
def delete_program_category(id):
    data = ProjectCategory.query.filter_by(id=id).first()
    db.session.delete(data)
    db.session.commit()
    flash('Deleted successfully', 'info')
    return redirect(url_for('program_add_category'))
  
@login_required
def publish_program_category():
    data = ProjectCategory.query.filter_by(id=id).first()
    x = 'Unpublished' if data.active else 'Published'
    data.active = False if data.active else True
    db.session.commit()
    
    flash(f'{x} successfully', 'info')
    return redirect(url_for('program_add_category'))
    
@login_required
def program_add_category():
    form = ProjectCategoryForm()
    if form.validate_on_submit():
        print(request.form)
        new =  ProjectCategory(
            name = form.name.data,
            description = form.description.data
        )
        db.session.add(new)
        db.session.commit()
        flash('Category added successfully', 'success')
    categories = ProjectCategory.query.filter_by(active=True).all()
    return render_template('admin/noredge/program_add_category.html', form=form, categories = categories)

@login_required
def program_add():
    form = ProjectForm()
    if request.method == "POST":
        if form.validate_on_submit():
            try:
                new = Projects(
                    name=form.name.data,
                    category = form.category.data,
                    description=form.description.data,
                    icon = form.icon.data
                )
                db.session.add(new)
                db.session.commit()
                flash('Project added succefully', 'success')
            except Exception as e:
                flash('Error adding project', 'error')
    form.category.choices = [(select.id, select.name) for select in ProjectCategory.query.filter_by(active=True).all()]
    return render_template('admin/noredge/program_add.html', form=form)


@login_required
def publish_program_item(id):
    data = Projects.query.filter_by(id=id).first()
    data.active = False if data.active else True
    db.session.commit()
    # cleanup(db.session)
    return redirect(url_for('program_items'))


@login_required
def delete_program_item(id):
    data = Projects.query.filter_by(id=id).first()
    db.session.delete(data)
    db.session.commit()
    # cleanup(db.session)
    return redirect(url_for('program_items'))

@login_required
def publish_activity_item(id):
    data = Activity.query.filter_by(id=id).first()
    data.active = False if data.active else True
    db.session.commit()
    # cleanup(db.session)
    return redirect(url_for('activity_items'))

@login_required
def delete_activity_item(id):
    data = Activity.query.filter_by(id=id).first()
    db.session.delete(data)
    db.session.commit()
    # cleanup(db.session)
    return redirect(url_for('activity_items'))


@login_required
def program_items():
    items = Projects.query.all()
    if request.method == 'POST':
        project = Projects.query.filter_by(id = request.form.get('project_id')).first()
        project.images = list(set(project.images + request.form.getlist('images')))
        db.session.commit()
    return render_template('admin/noredge/program_items.html', items=items)

@login_required
def activity_requests():
    items = db.session.query(ActivityRequests.name, 
                             ActivityRequests.id, 
                             ActivityRequests.email, 
                             ActivityRequests.phone,
                             ActivityRequests.date,
                             ActivityRequests.active,
                             Activity.name.label('activity_name'))\
                                .outerjoin(Activity, ActivityRequests.activity == Activity.id)\
                                .order_by(ActivityRequests.id.desc())\
                                .all()
    # if request.method == 'POST':
    #     project = Activity.query.filter_by(id = request.form.get('project_id')).first()
    #     project.images = list(set(project.images + request.form.getlist('images')))
    #     db.session.commit()
    return render_template('admin/noredge/activities/activity_requests.html', items=items)


@login_required
def program_payments():
    return render_template('admin/noredge/program_payments.html')
# =====================

# Admin blog views


@login_required
def blog_post_publish(id):
    data = Posts.query.filter_by(id=id).first()
    data.active = False if data.active else True
    db.session.commit()
    # cleanup(db.session)
    if data.active:
        flash(f'Item activated successfully', 'info')
    else:
        flash(f'Item deactivated successfully', 'info')
    return redirect(url_for('blog_posts'))


@login_required
def blog_post_delete(id):
    print(id)
    data = Posts.query.filter_by(id=id).first()
    db.session.delete(data)
    db.session.commit()
    # cleanup(db.session)
    flash(f'Item deleted successfully', 'info')
    return redirect(url_for('blog_posts'))


@login_required
def blog_posts():
    posts = db.session.query(Posts.id, Posts.active, Posts.keywords,
                             Posts.title, Posts.slug, Posts.image, Posts.upload_date, Posts.content,
                             Users.email).outerjoin(Users, Users.id == Posts.user_id).order_by(Posts.id.desc()).all()
    # cleanup(db.session)
    return render_template('admin/noredge/blog/blog_posts.html', posts=posts)


@login_required
def post_article():
    form = BlogForm()
    if request.method == "POST":
        if form.validate_on_submit():
            try:
                new = Posts(
                    title=form.title.data,
                    content=form.message.data,
                    image=form.image.data,
                    keywords=request.form.getlist('keywords'),
                    user_id=current_user.id
                )
                db.session.add(new)
                db.session.commit()
                flash('Article posted successfully', 'success')
            except Exception as e:
                print(e)
                flash('Error posting article', 'error')
            # finally:
                # cleanup(db.session)
    return render_template('admin/noredge/blog/post_article.html', form=form)
# ======================


def blog_images():
    images_list = []

    files = sorted(glob.iglob(os.path.join('static/blog', '*')),
                   key=os.path.getctime, reverse=True)
    file = [os.path.basename(x) for x in files]

    images_list = file[0:18]

    return make_response(jsonify({'images': images_list}))
# services admin views


@login_required
def service_delete(id):
    data = AllServices.query.filter_by(id=id).first()
    db.session.delete(data)
    db.session.commit()
    # cleanup(db.session)
    flash(f'Item deleted successfully', 'info')
    return redirect(url_for('all_services'))


@login_required
def all_services():
    services = db.session.query(AllServices.name,
                                AllServices.id,
                                AllServices.icon,
                                AllServices.service_category,
                                AllServices.description,
                                AllServices.upload_date,
                                ServiceCategories.name.label('category_name'),
                                ServiceCategories.icon.label('category_icon'))\
        .outerjoin(ServiceCategories, ServiceCategories.id == AllServices.service_category)\
        .order_by(AllServices.id.desc())\
        .all()
    # cleanup(db.session)
    for s in services:
        print(s.name)
    return render_template('admin/noredge/services/all_services.html', services=services)


@login_required
def add_service():
    form = ServiceForm()
    form.category.choices = [(select.id, select.name) for select in db.session.query(
        ServiceCategories.id, ServiceCategories.name).filter(ServiceCategories.active == True).order_by(ServiceCategories.id.desc()).all()]

    if request.method == 'POST':
        if form.validate_on_submit():
            print(request.form)
            try:
                new = AllServices(
                    name=request.form.get('name'),
                    icon=request.form.get('icon'),
                    service_category=request.form.get('category'),
                    slug=unique_slug(AllServices, request.form.get('name')),
                    description=request.form.get('description')
                )
                db.session.add(new)
                db.session.commit()
                flash('Service saved succesfully', 'success')
            except Exception as e:
                flash(e, 'error')
    return render_template('admin/noredge/services/add_service.html', form=form)


@login_required
def upload_files():
    image = request.values.get('image_upload')
    name = request.values.get('name')

    name = base64ToPNG(bytes(image, encoding='utf-8'), name)
    return make_response(jsonify({'name': name, 'message': "Files Uploaded successfully", "success": True}))


@login_required
def delete_service_categories(id):
    data = ServiceCategories.query.filter_by(id=id).first()
    db.session.delete(data)
    db.session.commit()
    # cleanup(db.session)
    flash(f'Item deleted successfully', 'info')
    return redirect(url_for('service_categories'))


@login_required
def publish_service_categories(id):
    data = ServiceCategories.query.filter_by(id=id).first()
    data.active = False if data.active else True
    db.session.commit()
    # cleanup(db.session)
    if data.active:
        flash(f'Item activated successfully', 'info')
    else:
        flash(f'Item deactivated successfully', 'info')
    return redirect(url_for('service_categories'))


@login_required
def service_categories():
    form = CategoryForm()
    if form.validate_on_submit():
        if request.method == "POST":
            try:
                new = ServiceCategories(
                    name=form.name.data,
                    icon=form.icon.data,
                    slug=unique_slug(ServiceCategories, form.name.data),
                    description=form.description.data
                )
                db.session.add(new)
                db.session.commit()
                flash('Service category saved succesfully', 'success')
            except Exception as e:
                flash(e, 'error')
    all = db.session.query(ServiceCategories).order_by(
        ServiceCategories.id.desc()).all()
    # cleanup(db.session)
    return render_template('admin/noredge/services/service_categories.html', form=form, cats=all)
# =======================

# Logout view


@login_required
def logout():
    logout_user()
    return redirect(url_for("login"))
# =======================
